@@ -1,5 +1,5 @@
 from gokart.pandas_type_config import PandasTypeConfig
-from gokart.parameter import TaskInstanceParameter, ListTaskInstanceParameter
+from gokart.parameter import TaskInstanceParameter, ListTaskInstanceParameter, ExplicitBoolParameter
 from gokart.task import TaskOnKart
 from gokart.info import make_tree_info, tree_info
 from gokart.testing import test_run
